/*********************
Pyrone Blog styles
**********************/
body {
	margin: 0;
	padding: 0;
}
body, div, table {
	font-family: "Liberation Sans", Tahoma, Verdana, sans-serif;
}
a {
	color: #0000ff;
	text-decoration: underline;
}
a:hover {
	color: #0040ff;
	text-decoration: underline;
}
a.border-icon {
	border: 1px solid #0000ff;
	text-decoration: none;
	color: #0000ff;
	background-color: #e5e5e5;
	padding-left: 3px; 
	padding-right: 3px; 
}
a.confirm-icon {
	border: 1px solid #800000;
	background-color: #800000;
	color: #fff;
	text-decoration: none;
}
span.notify-icon {
	border: 1px solid green;
	background-color: green;
	color: #fff;
	text-decoration: none;
}
a.new-window {
	text-decoration: none;
	border: 2px #0000ff;
	border-style: none none solid none;
}
.account-twitter {
	background-repeat: no-repeat;
	background-position: top left;
	background-image: url(images/twitter-icon.png);
	padding-left: 17px;
}
.account-admin {
	font-weight: bold;
	color: #060;
}
div#topbar, div#topbar-admin {
	position: absolute;
	width: 100%;
	margin: 0;
	height: 50px;
}
div#topbar {
	background-color: #f6f6ee;
}
div#topbar-admin {
	background-color: #ffd6a0;
}

div#topbar div.title, div#topbar div.title a,
div#topbar-admin div.title, div#topbar-admin div.title a {
	color: #6f0c0c;
	font-family: serif;
	font-size: 30px;
	font-weight: bolder;
	margin: 10px;
	text-decoration: none;
}
div#topbar-admin div.title {
	color: #a17170;
}
div#contentbar {
	position: absolute;
	margin-top: 50px;
	margin-right:280px;
	width: 100%;
}
div#content {
	padding: 15px;
	padding-right: 280px;
}
div#sidebar {
	border: dotted #f6f6ee;
	border-width: 0 0 3px 3px;
	position: absolute;
	width: 255px;
	background-color: #fff;
	right: 0;
	top: 50px;
	padding: 15px 0 10px 15px;
}
div#sidebar div.section {
	font-weight: bolder;
	border: dotted #f6f6ee;
	border-width: 2px 0 0 0;
	margin: 10px 0 5px 0;
	padding-top: 5px;
}
div#sidebar div.widget {
}
div.lang-selector {
	margin-top: 10px;
	border: 2px solid #f6f6ee;
	padding-left: 5px;
}
div.lang-selector span, div.lang-selector a {
	color: #BDBDB7;
}
div.lang-selector span.active {
	font-weight: bold;
}
div#bottombar {
	margin-top: 30px;
	border: dotted #f6f6ee;
	border-width: 3px 0 0 0;
}
div#bottombar div.title{
	margin: 4px 0 0 20px;
	font-size: 10pt;
	color: #7f7f7f;
}
div#bottombar a {
	color: #7f7f7f;
}

/* articles styles */
div.article-preview {
	padding: 10px;
	margin: 10px;
}
div.article-preview-after {
	height: 1px;
	border-style: dotted none none none;
	border-color: #f6f6ee;
	border-width: 3px;
	margin: 0 -5px 0 -5px;
}
div.article-preview div.title {
	padding: 5px 5px 5px 0;
	font-size: 17pt;
	font-weight: bolder;
}
div.article-preview div.tags {
	color: #989898;
	margin: 10px 0 0 0;
}
div.article-preview div.splitter {
	margin: 10px 0 10px 0;
}
div.article-preview div.before-preview {
	height: 2px;
	border: 1px #808080;
	border-style: dotted none none none;
}
div.article-preview div.after-preview {
	height: 2px;
	border: 1px #808080;
	border-style: none none dotted none;
}
div.article-preview div.tags a {
	color: #989898;
}
div.article-preview div.footer {
	color: #989898;
}
div.article-preview div.date {
	color: #989898;
}
div.pager {
	font-size: 17pt;
	font-weight: bolder;
	color: #7f7f7f;
}

div.tags-cloud {
}

a.tag-0 { font-size: 9px; color: #aaf;}
a.tag-5 { font-size: 10px; color: #aaf;}
a.tag-10 { font-size: 11px; color: #99f;}
a.tag-15 { font-size: 13px; color: #99f;}
a.tag-20 { font-size: 14px; color: #88f;}
a.tag-25 { font-size: 16px; color: #88f;}
a.tag-30 { font-size: 17px; color: #77f;}
a.tag-35 { font-size: 19px; color: #77f;}
a.tag-40 { font-size: 20px; color: #66f;}
a.tag-45 { font-size: 22px; color: #66f;}
a.tag-50 { font-size: 23px; color: #55f;}
a.tag-55 { font-size: 25px; color: #55f;}
a.tag-60 { font-size: 26px; color: #44f;}
a.tag-65 { font-size: 28px; color: #44f;}
a.tag-70 { font-size: 29px; color: #33f;}
a.tag-75 { font-size: 31px; color: #33f;}
a.tag-80 { font-size: 32px; color: #22f;}
a.tag-85 { font-size: 34px; color: #22f;}
a.tag-90 { font-size: 35px; color: #11f;}
a.tag-95 { font-size: 37px; color: #11f;}
a.tag-100 { font-size: 38px; color: #00f;}

/* article styles */
div.article div.title {
	padding: 5px 5px 5px 0;
	font-size: 17pt;
	font-weight: bolder;
}
div.article div.date {
	font-weight: bolder;
}
div.markup-preview {
	border: 1px solid #7f7f7f;
	padding: 5px;
}
div.article div.tags {
}
form#comment-form {
	background-color: #e5e5e5;
	padding: 0 5px 5px 5px;
}
div.article-inline-comment-edit textarea.body {
	width: 100%;
	height: 100px;
}
div.article-inline-comment-edit input[type=text] {
	border: 1px solid #A2A2A2;
}
div.article-inline-comment-edit input.display-name {
	width: 150px;
}
div.article-inline-comment-edit input.ip {
	width: 100px;
}
div.article-inline-comment-edit input.date {
	width: 150px;
	}
div.inline-form {
	border: 2px solid green;
	padding: 5px;
}
/* comment styles */
div.article-comment {
	border: 1px solid #d4d4d4;
	padding: 10px;
	margin: 4px 0 4px 0;
}
div.article-comment div.title {
	color: #989898;
}
div.article-comment div.title .name {
	font-weight: bolder;
}
div.article-comment div.body {
	margin: 10px 0 10px 0;
}

div.article-comments div.not-approved {
	border: 1px solid red;
}
div.article-comments div.active {
	border: 1px solid #00f;
}
span.comment-ip-address {
	font-style: italic;
}
div.comments-moderation-queue div.comment {
	border: 1px solid #a0a0a0;
	width: 100%;
	margin: 10px 0 0 0;
	padding: 5px 0 5px 5px;
}


input[type=text], input[type=password] {
	border: 1px solid #A2A2A2;
}
dl.form dt {
	margin: 10px 0 2px 0;
}
dl.form dd {
	margin: 4px 0 2px 30pt;
}
dl.form div.error {
	background-color: #ffd3d3;
	border: 1px dotted red;
	padding: 5px;
}
dl.form input.error, dl.form textarea.error {
	background-color: #ffd3d3;
}
dl.form input[type=text], dl.form input[type=password]{
	width: 95%;
}
dl.form textarea {
	width: 95%;
	height: 400px;
	border: 1px solid #A2A2A2;
}
dl.form textarea.small {
	width: 95%;
	height: 100px;
}

table.items-list {
	margin: 10px 0 10px 0;
	border: 1px solid #a52a2a;
	x-width: 100%;
}
table.items-list th {
	background-color: #d2d2d2;
	padding: 2px 10px 2px 10px;
}
table.items-list td {
	padding: 2px 5px 2px 5px;
}
span.hint, div.hint {
	color: #4d4d4d;
}
div.notify {
	border: 2px solid #009c00;
	padding: 10px;
	background-color: #dbffdb;
	color: #000;
	font-weight: bold;
}
div.warning {
	color: #a52a2a;
	border: 2px solid #a52a2a;
	padding: 10px;
	background-color: #fff9bf;
	color: #000;
	font-weight: bold;
}
span.warning {
	color: #a52a2a;
	border: 1px solid #a52a2a;
	padding: 0 2px 0 2px;
}
span.info {
	color: #005400;
	border: 1px solid #005400;
	padding: 0 2px 0 2px;
	cursor: default;
}
.hidden {
	display: none !important;
}
.strong {
	font-weight: bold;
}
.em-green {
	border: 1px solid green;
	background-color: #D2FAEA;
}
